#include <windows.h>
#include <windowsx.h>
#include <stdio.h>
#include <ras.h>
#include <raserror.h>
#include <string.h>
#include "ras.h"
#include "rc/resource.h"
#include "./lsapi.h"

#include <list>
using namespace std;

HWND hWnd;
HWND parent;
HINSTANCE hInstance;
HRASCONN hRasConn = NULL;
NOTIFYICONDATA systrayData;
list<dialEntry> entries;
char *szClassName = "LSDialer";
char s[30] = "";
RECT dialrect;
BOOL showDialog;
RASCONNSTATE rasConnState;

void addicon()
{
	char tooltip[64] = "Connected";
	systrayData.cbSize = sizeof(systrayData);
	systrayData.hIcon = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_TRAY)); 
	systrayData.hWnd = hWnd;
	lstrcpyn(systrayData.szTip, tooltip, 64);
	systrayData.uCallbackMessage = WM_TRAY; 
	systrayData.uFlags = NIF_ICON | NIF_MESSAGE | NIF_TIP ; 

	Shell_NotifyIcon(NIM_ADD , &systrayData);
}

void delicon()
{
	Shell_NotifyIcon(NIM_DELETE, &systrayData);
}

// called by RasDial()
VOID WINAPI RasDialFunc( UINT unMsg, RASCONNSTATE rasconnstate, DWORD dwError )
{
	rasConnState = rasconnstate;
	SendMessage(hWnd, WM_RASSTATE, 0, 0);
}

bool isConnected()
{
    LPRASCONN lpRasConn = NULL;
	LPRASCONN  lpTemp = NULL;
    LPHRASCONN lphTemp = NULL;
    DWORD cbBuf = 0;
    DWORD cConn = 0;
    DWORD dwRet = 0;

	if ((lpRasConn = ( LPRASCONN ) _fmalloc((UINT)cbBuf)) != NULL )
	{
		cbBuf = sizeof(RASCONN);
		lpRasConn->dwSize = sizeof( RASCONN );
		dwRet = RasEnumConnections( lpRasConn, &cbBuf, &cConn );

        if ( dwRet == ERROR_BUFFER_TOO_SMALL )
        {
            if ( (lpTemp = ( LPRASCONN ) _frealloc(lpRasConn, (UINT)cbBuf) ) != NULL )
            {
                lpRasConn = lpTemp;
                dwRet = RasEnumConnections( lpRasConn, &cbBuf, &cConn );
            }
            else
            {
                dwRet = ERROR_NOT_ENOUGH_MEMORY;
            }
        }
        else if ( dwRet ) // other error
        {
			char  szMessage[256];
            if ( RasGetErrorString( (UINT)dwRet, szMessage, 256 ) != 0 )
                wsprintf( (LPSTR)szMessage, (LPSTR)"Undefined RAS Dial Error." );

            MessageBox(NULL, szMessage, "Error", MB_OK | MB_ICONSTOP );                            
        }

		_ffree( lpRasConn );

		if (!dwRet && cConn)
		{
			return true;
		}
	}

	return false;
}

void HangupCur()
{
	if (hRasConn)
	{
		RasHangUp( hRasConn );
		hRasConn = NULL;
	}
}

void Hangup()
{
	HangupCur();

    LPRASCONN lpRasConn = NULL;
	LPRASCONN  lpTemp = NULL;
    LPHRASCONN lphTemp = NULL;
    DWORD cbBuf = 0;
    DWORD cConn = 0;
    DWORD dwRet = 0;

	if ((lpRasConn = ( LPRASCONN ) _fmalloc((UINT)cbBuf)) != NULL )
	{
		cbBuf = sizeof(RASCONN);
		lpRasConn->dwSize = sizeof( RASCONN );
		dwRet = RasEnumConnections( lpRasConn, &cbBuf, &cConn );

        if ( dwRet == ERROR_BUFFER_TOO_SMALL )
        {
            if ( (lpTemp = ( LPRASCONN ) _frealloc(lpRasConn, (UINT)cbBuf) ) != NULL )
            {
                lpRasConn = lpTemp;
                dwRet = RasEnumConnections( lpRasConn, &cbBuf, &cConn );
            }
            else
            {
                dwRet = ERROR_NOT_ENOUGH_MEMORY;
            }
        }
        else if ( dwRet != 0 ) // other error
        {
			char  szMessage[256];
            if ( RasGetErrorString( (UINT)dwRet, szMessage, 256 ) != 0 )
                wsprintf( (LPSTR)szMessage, (LPSTR)"Undefined RAS Dial Error." );

            MessageBox(NULL, szMessage, "Error", MB_OK | MB_ICONSTOP );                            
        }

		if (!dwRet)
		{
			RasHangUp(lpRasConn[0].hrasconn);
		}

		_ffree( lpRasConn );
	}
}

void HangupBang(HWND caller, const char* arg)
{
	delicon();
	DestroyWindow(hWnd);
	Hangup();
}

LRESULT CALLBACK mainWndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	HDC hDC;
	PAINTSTRUCT PaintSt;
	RECT r;

	switch(message)
	{
	case WM_CREATE:	return 0;
	case WM_TRAY:
		{
			switch (lParam)
			{
				case WM_LBUTTONDBLCLK:
					{
						Hangup();
						delicon();
						DestroyWindow(hWnd);
						return 0;
					}
				default: return 0;
			}
		}
	case WM_PAINT:
		{
			hDC = BeginPaint(hWnd, &PaintSt);
			GetClientRect(hWnd, &r);
			SetBkMode(hDC, TRANSPARENT);
			DrawText(hDC, s, -1, &r, DT_CENTER);
			EndPaint(hWnd, &PaintSt);
			return 0;
		}
	case WM_CLOSE:
		{
			Hangup();
			DestroyWindow(hWnd);
			return 0;
		}
	case WM_DESTROY:
		{
			return 0;
		}
	case WM_RASSTATE:
		{
			GetRasConnState( rasConnState, s );

			if (!strcmp(s, "Connected"))
			{
				addicon();
				ShowWindow(hWnd, SW_HIDE);
			}
			else if (!strcmp(s, "Disconnected"))
			{
				DestroyWindow(hWnd);
				delicon();
			}
			InvalidateRect(hWnd, &dialrect, TRUE);

			return 0;
		}
	}
	return DefWindowProc(hWnd, message, wParam, lParam);
}

void Connect(HWND caller, const char* arg)
{
	RASDIALPARAMS rdParams;
	DWORD dwRet;
	char szBuf[256];
	char extra[256];
	dialEntry d;
	rdParams.dwSize = sizeof(RASDIALPARAMS);
	RECT r;
	GetClientRect(GetDesktopWindow(),&r);

	// litestep stuff
	int count;
	char* tokens[6];
	tokens[0] = d.phEntry;
	tokens[1] = d.username;
	tokens[2] = d.password;
	tokens[3] = d.domain;

	count = LCTokenize(arg, tokens, 4, extra);

	lstrcpy(rdParams.szEntryName, d.phEntry );
	lstrcpy(rdParams.szUserName, d.username );
	lstrcpy(rdParams.szPassword, d.password );
	lstrcpy(rdParams.szDomain, d.domain );

	if (!isConnected())
	{
		DestroyWindow(hWnd);
		hRasConn = NULL;
	}
	else
	{
		MessageBox(NULL, "Connection already open, try disconnecting first.",
			"Error", MB_OK);
		return;
	}
	
	hWnd = CreateWindow(szClassName, d.phEntry, WS_SYSMENU,
		(r.right - dialrect.right)/2, (r.bottom - dialrect.bottom)/2,
		dialrect.right, dialrect.bottom,
		0, 0, hInstance, 0);

	if (!hWnd)
		MessageBox(NULL, "Error creating window.", "error", MB_OK);

	if (showDialog)
		ShowWindow(hWnd, SW_SHOWNORMAL);

    dwRet = RasDial( NULL, NULL, &rdParams, 0L, (RASDIALFUNC) RasDialFunc, &hRasConn );

    if ( dwRet )
    {
        if ( RasGetErrorString( (UINT)dwRet, (LPSTR)szBuf, 256 ) != 0 )
            wsprintf( (LPSTR)szBuf, "Undefined RAS Dial Error (%ld).", dwRet );

        MessageBox(caller, (LPSTR)szBuf, "Error", MB_OK | MB_ICONSTOP );
		DestroyWindow(hWnd);
    }
}

void Entry(HWND caller, const char* arg)
{
	list<dialEntry>::const_iterator lci;
	bool found = false;
	for (lci = entries.begin(); lci != entries.end(); lci++)
	{
		if (!strcmp(arg, lci->name))
		{
			char params[1024];
			strcpy(params, "\"");
			strcat(params, lci->phEntry);
			strcat(params, "\" \"");
			strcat(params, lci->username);
			strcat(params, "\" \"");
			strcat(params, lci->password);
			strcat(params, "\" \"");
			strcat(params, lci->domain);
			strcat(params, "\"");

			Connect(caller, params);
			found = true;
		}
	}

	if (!found)
	{
		MessageBox(NULL, "Entry not found.", "Error", 
			MB_OK | MB_ICONSTOP);
	}
}

int initModule(HWND ParentWnd, HINSTANCE dllInst, wharfDataType* wd)
{
	return initModuleEx (ParentWnd, dllInst, wd->lsPath);
}

int initModuleEx(HWND ParentWnd, HINSTANCE dllInst, LPCSTR szPath)
{
	parent = ParentWnd;
	hInstance = dllInst;
	hRasConn = NULL;

	// register the dialing window
	WNDCLASS wc;
	ZeroMemory(&wc, sizeof(WNDCLASSEX));
	wc.lpfnWndProc = (WNDPROC)mainWndProc;
	wc.hInstance = hInstance;
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hIcon = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_SYSICON));
	wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
	wc.lpszClassName = szClassName;
	RegisterClass(&wc);

	dialrect.top = dialrect.left = 0;
	dialrect.right = 200;
	dialrect.bottom = 60;

	AddBangCommand("!DialerHangup", HangupBang);
	AddBangCommand("!DialerConnect", Connect);
	AddBangCommand("!DialerEntry", Entry);

	showDialog = GetRCBool("DialerNoDialog", FALSE);

	FILE *f;
	f = LCOpen(NULL);
	if (f)
	{
		int count;
		char buffer[256], garbage[50], extra_text[256];
		char* tokens[6];
		dialEntry temp;

		tokens[0] = garbage;
		tokens[1] = temp.name;
		tokens[2] = temp.phEntry;
		tokens[3] = temp.username;
		tokens[4] = temp.password;
		tokens[5] = temp.domain;

		while (LCReadNextConfig(f, "*LSDialerEntry", buffer, 
			sizeof (buffer)))
		{
			count = LCTokenize (buffer, tokens,6, extra_text);
			entries.push_back(temp);
		}
	}

	LCClose(f);

	if (isConnected())
	{
		hWnd = CreateWindow(szClassName, "hidden", WS_SYSMENU,
			0, 0, // x,y
			10, 10, // width, height
			0, 0, hInstance, 0);

		if (!hWnd)
			MessageBox(NULL, "Error creating window.", "error", MB_OK);

		addicon();
		ShowWindow(hWnd, SW_HIDE);
	}

	return 0;
}

void quitModule(HINSTANCE dllInst)
{
	entries.clear();
	DestroyWindow(hWnd);
	UnregisterClass(szClassName, dllInst);
	RemoveBangCommand("!DialerHangup");
	RemoveBangCommand("!DialerConnect");
	RemoveBangCommand("!DialerEntry");
}

